/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * cvtdate - convert POSIX touch date/time to seconds since epoch time
 *	
 *	datestr is a date/time string in the form: [[[[CC]YY]MM]DD]hhmm[.SS]
 *	as defined by POSIX.  
 *		CC = centry, ie 19 or 20
 *		YY = year, if CC is not provided and YY is < 69, then 
 *		     CC is assumed to be 20, else 19.
 *		MM = Month, [1,12], if YY is not provided and MM is less than
 *		     the current month, YY is next year, else it is the
 *		     current year.
 *		DD = Day of month, [1,31], if MM is not provided and DD is less
 *		     than the current day, MM is next month, else it is the
 *		     next month.
 *		hh = hour, [00, 23], if DD is not provided and hh is less than
 *		     the current hour, DD is tomorrow, else it is today.
 *		mm = minute, [00, 59]
 *		SS = seconds, [00, 59]
 *
 *	Returns: time_t number of seconds since epoch (Coordinated Univ. Time)
 *		 -1 if error.
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

static char ident[] = "@(#) $RCSfile: cvtdate.c,v $ $Revision: 2.0 $";

time_t
cvtdate(datestr)
char   *datestr;
{
	char	   buf[3];
	time_t	   clock;
	int	   i;
	struct tm *ptm;
	char      *pc;
	struct tm  tm;
	int	   year = 0;
	int	   month = -1;
	int	   day = 0;


	if (pc = strchr(datestr, (int)'.')) {
		*pc++ = '\0';
		if ((strlen(pc) != 2) ||
		    (isdigit((int)*pc) == 0) ||
		    (isdigit((int)*(pc+1)) == 0))
			return (-1);
		tm.tm_sec = atoi(pc);
		if ( tm.tm_sec > 59 )
			return (-1);
	} else
		tm.tm_sec = 0;

	for (pc = datestr; *pc; ++pc)
		if (isdigit((int)*pc) == 0)
			return (-1);

	buf[2] = '\0';
	clock = time((time_t *)0);
	ptm   = localtime(&clock);
	tm.tm_year = ptm->tm_year;	/* default year to current */
	tm.tm_mon  = ptm->tm_mon;	/* default month to current */
	tm.tm_mday = ptm->tm_mday;	/* default day to current */

	switch (strlen(datestr)) {

	    case 12:		/* CCYYMMDDhhmm */
		buf[0] = datestr[0];
		buf[1] = datestr[1];
		year   = atoi(buf) * 100;
		datestr += 2;

		/* no break, fall into next case */

	    case 10:		/* YYMMDDhhmm */
		buf[0] = datestr[0];
		buf[1] = datestr[1];
		i      = atoi(buf);
		if (year == 0)
			if (i > 68)
				year = 1900 + i;
			else
				year = 2000 + i;
	
		else
			year += i;
		tm.tm_year = year - 1900;
		datestr += 2;
		
		/* no break, fall into next case */

	    case 8:		/* MMDDhhmm */
		buf[0] = datestr[0];
		buf[1] = datestr[1];
		i = atoi(buf);
		if (i < 1 || i > 12)
			return (-1);
		if (year == 0)
			if (i <= ptm->tm_mon)
				tm.tm_year++;
		month = i - 1;
		tm.tm_mon = month;
		datestr += 2;
		
		/* no break, fall into next case */

	    case 6:		/* DDhhmm */
		buf[0] = datestr[0];
		buf[1] = datestr[1];
		day = atoi(buf);
		if (day < 1 || day > 31)
			return (-1);
		if (month == -1)
			if (day < ptm->tm_mday)
				tm.tm_mon++;
		tm.tm_mday = day;
		datestr += 2;
		
		/* no break, fall into next case */

	    case 4:		/* hhmm */
		buf[0] = datestr[0];
		buf[1] = datestr[1];
		tm.tm_hour = atoi(buf);
		if (tm.tm_hour > 23)
			return (-1);

		tm.tm_min = atoi(&datestr[2]);	/* mm -  minute portion */
		if (tm.tm_min > 59)
			return (-1);
		if (day == 0)			/* day not specified */
			if ( (tm.tm_hour < ptm->tm_hour) ||
			     ((tm.tm_hour == ptm->tm_hour) &&
			      (tm.tm_min <= ptm->tm_min)) )
				tm.tm_mday++;	/* time for tomorrow */

		break;

	    default:
		return (-1);
	}
		
	tm.tm_isdst = -1;

	return (mktime(&tm));
}
